import os
import shutil
import json
import requests
from datetime import datetime, timedelta

import schedule
from fastapi import FastAPI, Response, Request
from fastapi.responses import RedirectResponse, FileResponse, HTMLResponse
from bs4 import BeautifulSoup as Bs


app = FastAPI()

channel_urls = {"first_url": "https://tv.yandex.ru/channel/pervyy-16",
                "russia_url": "https://tv.yandex.ru/channel/rossiya-1-31",
                "match_url": "https://tv.yandex.ru/channel/match-tv-49",
                "ntv_url": "https://tv.yandex.ru/channel/ntv-11",
                "five_url": "https://tv.yandex.ru/channel/pyatyy-kanal-12",
                "culture_url": "https://tv.yandex.ru/channel/kultura-14",
                "rus24_url": "https://tv.yandex.ru/channel/rossiya-24-3",
                "karusel_url": "https://tv.yandex.ru/channel/karusel-20",
                "otr_url": "https://tv.yandex.ru/channel/obshchestvennoe-televidenie-rossii-51",
                "center_url": "https://tv.yandex.ru/channel/tv-centr-32",
                "ren_url": "https://tv.yandex.ru/channel/ren-30",
                "spas_url": "https://tv.yandex.ru/channel/spas-52",
                "sts_url": "https://tv.yandex.ru/channel/sts-8",
                "domashniy_url": "https://tv.yandex.ru/channel/domashniy-5",
                "tv3_url": "https://tv.yandex.ru/channel/tv-3-17",
                "friday_url": "https://tv.yandex.ru/channel/pyatnica-42",
                "star_url": "https://tv.yandex.ru/channel/zvezda-15",
                "world_url": "https://tv.yandex.ru/channel/mir-54",
                "tnt_url": "https://tv.yandex.ru/channel/tnt-33",
                "muz_url": "https://tv.yandex.ru/channel/muz-tv-55"}

channel_names = {"first_url": "Первый",
                 "russia_url": "Россия 1",
                 "match_url": "Матч!",
                 "ntv_url": "НТВ",
                 "five_url": "Пятый канал",
                 "culture_url": "Культура",
                 "rus24_url": "Россия 24",
                 "karusel_url": "Карусель",
                 "otr_url": "ОТР",
                 "center_url": "ТВ Центр",
                 "ren_url": "Рен ТВ",
                 "spas_url": "Спас",
                 "sts_url": "СТС",
                 "domashniy_url": "Домашний",
                 "tv3_url": "ТВ 3",
                 "friday_url": "Пятница",
                 "star_url": "Звезда",
                 "world_url": "МИР",
                 "tnt_url": "ТНТ",
                 "muz_url": "МУЗ ТВ"}


def save_image(url):
    """image = requests.get(url).content  # получение картинки
    name = f"source/{datetime.now()}/image.png"  # текущая дата используется для уникальности имени картинки
    with open(name, 'wb') as file:
        file.write(image)  # сохранение картинки
    return name  # возврат локальной ссылки"""
    return url


@app.get("/")
async def main_page():
    return RedirectResponse(f"channels/{datetime.now().strftime('%y-%m-%d')}")


@app.get("/select")
async def select_date():
    content = "<b>Select date</b><hr>"
    for i in reversed(sorted(os.listdir("storage"))):
        content += f"""<a href=/channels/{i}>{i}</a><hr>"""
    return HTMLResponse(content)


@app.get("/channels/{date}")
def get_channels(date: str):
    content = "<b>Select channel</b> <a href='/select'>Select other date</a><hr>"
    for i in reversed(sorted(os.listdir(f"storage/{date}"))):
        content += f"""<a href=/program/{date}/{i}>{i[:-5]}</a><hr>"""
    return HTMLResponse(content)


@app.get("/program/{date}/{channel}")
def get_channels(date: str, channel: str):
    with open(f"storage/{date}/{channel}", "r") as file:
        data = json.load(file)  # читаем файл
    content = "<b>Телепрограмма</b> <hr>"
    for i in data:
        content += f"<a href=/info/{date}/{channel}/{i['time']}><b>{i['time']}</b> {i['name']}</a><hr>"
    return HTMLResponse(content)


@app.get("/info/{date}/{channel}/{time}")
def get_info(date: str, channel: str, time: str):
    with open(f"storage/{date}/{channel}", "r") as file:
        data = json.load(file)  # читаем файл
    for i in data:  # проходим по программам
        if i['time'] == time:
            url = i['url']  # получаем ссылку на картинку
            name = i['name']  # имя программы для получения информации
    headers = {'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                             'Chrome/75.0.3770.142 Safari/537.36'}  # заголовки запроса
    request = requests.get(f"https://www.themoviedb.org/search?query={name}", headers=headers).text
    soup = Bs(request, "html.parser")
    soup = soup.find("div", {"class": "white_column"}).find("div", {"class": "results"}).find("a", {"class": "result"})
    local_url = soup['href']  # получаем ссылку на пекредачу
    request = requests.get(f"https://www.themoviedb.org{local_url}", headers=headers).text
    soup = Bs(request, "html.parser")  # получаем информацию о передаче
    _type = ""
    temp = str(soup.find("span", {"class": "genres"})).split(",")  # получаем категории передачи
    for i in temp:  # проходим по каждой категории
        _type += i.split('tv">')[1].split("</span>")[0][:-4] + ", "  # тип передачи
    temp = soup.find("div", {"class": "header_info"}).find("div", {"class": "overview"})
    description = soup.find("div", {"class": "header_info"}).find("div", {"class": "overview"}).text  # описание
    actors = soup.find("ol", {"class": "people"}).text  # актеры
    if "Вы можете помочь" in actors:  # если актеры не указаны
        actors = "Актеры не указаны"
    return HTMLResponse(f"""<img src="{url}" alt="Картинка программы"><hr>Тип: {_type[:-3]}<br>
                        Описание: {description}<br>Актеры: {actors}""")  # собираем страницу


def parse(date: str, channel: str):
    request = requests.get(f"{channel}?date={date}").text  # получение кода страницы с телепередачей
    soup = Bs(request, "html.parser")
    result = []
    for i in soup.find_all("li"):  # телепрограмма записана в теге li
        try:
            time = i.find("time").text  # получение времени программы
            name = i.find("span", {"class": "channel-schedule__text"}).text  # получение названия программы
            url = i.find("a")["href"]  # ссылка на картинку
            local_request = requests.get("https://tv.yandex.ru" + url).text  # ссылка на картинки
            local_soup = Bs(local_request, "html.parser").find("section", {"class": "program-gallery"}).find("meta")
            result.append({"time": time, "name": name, "url": save_image(str(local_soup)[15:-18])})
            # save_image() будет описан в пункте 2.1
        except TypeError:  # если нет картинки
            result.append({"time": time, "name": name, "url": "None"})
    return result


def cache():
    folders = os.listdir('storage')
    if datetime.now().strftime('%y-%m-%d') not in folders:
        shutil.rmtree(f"storage/{folders[0]}")  # удаление старой телепрограммы
        name = datetime.now().strftime('%y-%m-%d')
        os.mkdir(f"storage/{name}")  # создание новой папки для сегодняшнего расписания
        for j in channel_urls:  # проход по словарю, где записыны ссылки на каналы
            data = parse(datetime.now().strftime('%y-%m-%d'), channel_urls[j])  # получение телепрограммы
            with open(f"storage/{name}/{channel_names[j]}.json", "w") as file:
                json.dump(data, file, indent=2)  # создание .json


def create_date_folders():
    for i in range(14):  # количество дней, которые сохраняем
        try:
            name = (datetime.now() - timedelta(days=i)).strftime('%y-%m-%d')  # название папки
            os.mkdir(f"storage/{name}")  # создание папки
            for j in channel_urls:  # проходимся по каналам
                data = parse(datetime.now().strftime('%y-%m-%d'), channel_urls[j])  # получение телеперограммы канала
                with open(f"storage/{name}/{channel_names[j]}.json", "w") as file:
                    json.dump(data, file, indent=2)  # записываем в файл
        except FileExistsError:
            pass


@app.on_event("startup")
def startup():
    schedule.every().day.at("00:01").do(cache)
    cache()
    try:
        os.mkdir("storage")
    except FileExistsError:
        pass
    finally:
        create_date_folders()
